from aiohttp import ClientSession

class OpenRouterAPI:
    def __init__(self, api_key: str, model: str):
        self.api_key = api_key
        self.model = model

        self.urls = {
            "completion": "https://openrouter.ai/api/v1/completions",
            "chat_completion": "https://openrouter.ai/api/v1/chat/completions",
            "get_generation": "https://openrouter.ai/api/v1/generation",
            "get_list_of_models": "https://openrouter.ai/api/v1/models",
            "get_list_of_endpoints_for_model": "https://openrouter.ai/api/v1/models/author/slug/endpoints",
            "get_credits": "https://openrouter.ai/api/v1/credits"    
        }   

        self.headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.api_key}"
        }

        self.data = {
            "model": self.model,
            "message": [
                {
                    "role": "user",
                    "content": ""
                }
            ]
        }

    async def send_completion(self, prompt: str) -> dict:
        """Send a completion request to a selected model (text-only format)

        Args:
            prompt (str): The text prompt to complete

        Returns:
            dict: Successful completion response from the API
        """
        self.data["message"][0]["content"] = prompt

        async with ClientSession() as session:
            async with session.post(self.urls["completion"], headers=self.headers, json=self.data) as response:
                return await response.json()
            
    
    async def chat_completion(self, prompt: str) -> dict:
        """Send a chat completion request to a selected model

        Args:
            prompt (str): The text prompt to complete

        Returns:
            dict: Successful completion response from the API
        """
        self.data["messages"].append({"role": "user", "content": prompt})

        async with ClientSession() as session:
            async with session.post(self.urls["chat_completion"], headers=self.headers, json=self.data) as response:
                answer = await response.json()

                return answer
            
    
    async def get_generation(self, generation_id: str) -> dict:
        """Returns metadata about a specific generation request

        Args:
            generation_id (str): _description_

        Returns:
            dict: Returns the request metadata for this generation
        """
        async with ClientSession() as session:
            async with session.get(f"{self.urls['get_generation']}/{generation_id}", headers=self.headers) as response:
                return await response.json()
            
    
    async def get_list_of_models(self) -> dict:
        """Returns a list of models available through the API

        Returns:
            dict: List of available models
        """
        async with ClientSession() as session:
            async with session.get(self.urls["get_list_of_models"], headers=self.headers) as response:
                return await response.json()
            
    
    async def get_list_of_endpoints_for_model(self) -> dict:
        """Returns a list of endpoints for a specific model

        Returns:
            dict: List of endpoints for the model
        """
        async with ClientSession() as session:
            async with session.get(self.urls["get_list_of_endpoints_for_model"], headers=self.headers) as response:
                return await response.json()
            

    async def get_credits(self) -> dict:
        """Returns the total credits purchased and used for the authenticated user

        Returns:
            dict: Returns the total credits purchased and used
        """
        async with ClientSession() as session:
            async with session.get(self.urls["get_credits"], headers=self.headers) as response:
                return await response.json()